// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//     Fichier contenant les routines de gestion EEPROM
//
//  Ces fonctions utilisent les routines de la librairie avr-libc :
//       uint8_t eeprom_read_byte (const uint8_t *addr)
//       uint16_t eeprom_read_word (const uint16_t *addr)
//       void eeprom_write_byte (uint8_t *addr, uint8_t val)
//       void eeprom_write_word (uint16_t *addr, uint16_t val)
//=======================================
// History:
// 24/10/04    V1.0
//       Version originale
// 03/11/06    V1.1
//       mise en commanetaires des traces de debug
//=======================================


#include "TIPLL_Eeprom.h"
#include "TIPLL_USART2.h"

/* include system */
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>   

/* declaration des variables static */
static  T_TIPLL_EEPROM_data  * pt_EEPROM_Data;  /* pointeur sur la structure EEPROM */





/* fonctions applicatives de gestion EEPROM */


/* initialisation pointeur, controle validite et init par defaut en cas de donnes non valides */
void EEP_Init(void)
{
    char ResultCheck;    /* resultat check EEPROM */
	
    /* init pointeur sur la structure de donnees situee a la fin de memoire  EEPROM delimite par E2END */
    pt_EEPROM_Data = (T_TIPLL_EEPROM_data  *) (E2END - sizeof( T_TIPLL_EEPROM_data));
	
	/* controle validite EEPROM */
	ResultCheck = EEP_Check();
	//debug USART_PrintfStr(PSTR("EEP_Init: ResultCheck="));
	//debug USART_PrintfUi_IT((unsigned int)ResultCheck,0);
	//debug USART_PrintfStr(PSTR("\r\n"));

	/* SI donnees non valides */
	/* ALORS */
	if (ResultCheck != 0)
	{
	    /* init par defaut EEPROM */
		EEP_SetDefault();
	/* FINSI */
	}
}


/* initialisation par defaut de l'EEPROM */
void EEP_SetDefault(void)
{
   	// debug USART_PrintfStr(PSTR("EEP_SetDefault\r\n")); 
    eeprom_write_word(&(pt_EEPROM_Data->FreqDefault),EEP_FREQ_DEF_VAL);
    eeprom_write_word(&(pt_EEPROM_Data->FreqQuick[0]),EEP_FREQ1_VAL);
    eeprom_write_word(&(pt_EEPROM_Data->FreqQuick[1]),EEP_FREQ2_VAL);
    eeprom_write_word(&(pt_EEPROM_Data->FreqQuick[2]),EEP_FREQ3_VAL);
    eeprom_write_word(&(pt_EEPROM_Data->Prog_FreqDefault),EEP_PROG_VAL);
	
	/* calcul validite data */
	EEP_update_valid();   /* dans ce cas la valeur retournee n'est pas utilisee*/
}

/* controle la validite de l'EEPROM , retourne 0 si OK, -1 si KO */
char EEP_Check(void)
{ 
    char Result;
	unsigned int TmpCheckSum;  /* valeur temporaire de check sum */
    unsigned int Index;        /* index de boucle calcul checkSum */	
	uint8_t * pt_EEP_data;      /* pointeur temporaire donnees EEPROM */
	


	Result=0;   /* par defaut OK */
	TmpCheckSum=0;
	pt_EEP_data = (uint8_t *) pt_EEPROM_Data;
	
	/* boucle de calcul checksum sans comptabiliser le controle de validite  */
	for (Index = (sizeof(T_TIPLL_EEPROM_data) -sizeof(uint16_t));Index !=0; Index--)
	{
	     TmpCheckSum += eeprom_read_byte(pt_EEP_data);
		 pt_EEP_data++;
	}
	
	/* debug */
   	//debug USART_PrintfStr(PSTR("EEP_Check : TmpCheckSum="));
    //debug USART_PrintfUi_IT( TmpCheckSum,1);
    //debug USART_PrintfStr(PSTR("\r\n"));
   	//debug USART_PrintfStr(PSTR("EEP_Check : Validite_EEPROM="));
    //debug USART_PrintfUi_IT( pt_EEPROM_Data->Validite_EEPROM,1);
   	//debug USART_PrintfStr(PSTR("\r\n"));
	

	if ( TmpCheckSum != eeprom_read_word(&(pt_EEPROM_Data->Validite_EEPROM)))
	{
	   Result =-1;
	}
	return Result;
}

/* memorisation Frequence par defaut */
void EEP_set_FreqDefault(uint16_t FreqDef)
{
    eeprom_write_word(&(pt_EEPROM_Data->FreqDefault),FreqDef);
	/* calcul validite data */
	EEP_update_valid();   /* dans ce cas la valeur retournee n'est pas utilisee*/

}

/* memorisation Frequence rapide */
void EEP_set_FreqQuick(unsigned char Indice,uint16_t FreqDef)
{
	eeprom_write_word(&(pt_EEPROM_Data->FreqQuick[Indice]),FreqDef);
    /* calcul validite data */
	EEP_update_valid();   /* dans ce cas la valeur retournee n'est pas utilisee*/
}

/* memorisation flag de programmation initiale */
void EEP_set_FlagProg(uint16_t Flag)
{
    eeprom_write_word(&(pt_EEPROM_Data->Prog_FreqDefault),Flag);
	/* calcul validite data */
	EEP_update_valid();   /* dans ce cas la valeur retournee n'est pas utilisee*/
}

/* retourne la Frequence par defaut */
uint16_t EEP_get_FreqDefault(void)
{
    return (eeprom_read_word(&(pt_EEPROM_Data->FreqDefault)));
}

/* retourne la  Frequence rapide */
uint16_t EEP_get_FreqQuick(unsigned char Indice)
{
    return (eeprom_read_word(&(pt_EEPROM_Data->FreqQuick[Indice])));
}

/* retroune le flag de programmation initiale */
uint16_t EEP_get_FlagProg(void)
{
    return (eeprom_read_word(&(pt_EEPROM_Data->Prog_FreqDefault)));
}

/* calcule la checksum de validite, la memorise et la retourne */
uint16_t EEP_update_valid(void)
{
	uint16_t TmpCheckSum;        /* valeur temporaire de check sum */
    unsigned int Index;        /* index de boucle calcul checkSum */	
	uint8_t * pt_EEP_data;      /* pointeur temporaire donnees EEPROM */
	
	
	TmpCheckSum=0;
	pt_EEP_data = (uint8_t *) pt_EEPROM_Data;
	
	/* boucle de calcul checksum  sans comptabiliser le controle de validite */
	for (Index = (sizeof(T_TIPLL_EEPROM_data) -sizeof(uint16_t));Index !=0; Index--)
	{
	     TmpCheckSum += eeprom_read_byte(pt_EEP_data);
		 pt_EEP_data++;
	}
   	//debug USART_PrintfStr(PSTR("EEP_update_valid : TmpCheckSum="));
   	//debug USART_PrintfUi_IT(TmpCheckSum,1);
   	//debug USART_PrintfStr(PSTR("\r\n"));
	/* update de la checksum obtenue */
    eeprom_write_word(&(pt_EEPROM_Data->Validite_EEPROM),TmpCheckSum);

    return (TmpCheckSum);
}







